@@ -139,9 +139,10 @@ class TDeviceWin(TDeviceBase) :
         return this.__SendCtrlCmd('BACK')
 
     def ReadReg(this, regAddr, byteOffset, readCount, regValueList, regValueListStartIdx=0) :
+        dutIdx = 0
         regValueInt = 0
         regValueByteArray=bytes(readCount)
-        ret = this.__Api.TDeviceDD_DSIM_MipiReadReg(this.__DeviceHandle, regAddr, byteOffset, readCount, regValueByteArray)
+        ret = this.__Api.TDeviceDD_DSIM_MipiReadReg(this.__DeviceHandle, dutIdx, regAddr, byteOffset, readCount, regValueByteArray, readCount)
         if ret==1 :
             for idx, regValueByte in enumerate(regValueByteArray) :
                 regValueInt = regValueByte
@@ -153,14 +154,8 @@ class TDeviceWin(TDeviceBase) :
         return bflag
 
     def ReadReg1Byte(this, regAddr, byteOffset) :
-        regValueArray=bytes(1)
-        ret = this.__Api.TDeviceDD_DSIM_MipiReadReg1Byte(this.__DeviceHandle, regAddr, byteOffset, regValueArray)
-        if ret==1 :
-            regValue = regValueArray[0]
-            regValue &= 0xFF
-        else :
-            regValue = -1
-        return regValue
+        dutIdx = 0
+        return this.__Api.TDeviceDD_DSIM_MipiReadReg1Byte(this.__DeviceHandle, dutIdx, regAddr, byteOffset)
 
     def WriteReg(this, regAddr, byteOffset, writeCount, regValueList, writeDataStartIdx=0) :
 
@@ -187,8 +182,6 @@ class TDeviceWin(TDeviceBase) :
 
     def WriteCtrlReg(this, regAddr) :
         return this.WriteReg1Byte(regAddr, 0, 1)
-
-
     
     def CatchPower(this, powerInfo) :
 
@@ -638,27 +631,45 @@ class TDeviceWin(TDeviceBase) :
         return 1 #ted.AdcSoutGetRBSel(dutIdx)
 
 
-    #COMM_API Bool TedDD_DSIM_MipiReadReg(int dutIdx, int addr, int byteOffset, int readCount, unsigned char* buf, int bufMaxByteSize);
+    #TDEVICE_API TED_BOOL TDeviceDD_DSIM_MipiReadReg(void* hdl, int dutIdx, int addr, int byteOffset, int readCount, unsigned char* buf, int bufMaxByteSize);
     def DD_DSIM_MipiReadReg(this, dutIdx, regAddr, byteOffset, readCount) : 
-        return 1 #ted.DD_DSIM_MipiReadReg(dutIdx, regAddr, byteOffset, readCount)
 
-    #COMM_API unsigned char TedDD_DSIM_MipiReadReg1Byte(int dutIdx, int addr, int byteOffset);
+        regValueList=[0 for _ in range(readCount)]
+        regValueByteArray=bytes(readCount)
+        ret = this.__Api.TDeviceDD_DSIM_MipiReadReg(this.__DeviceHandle,  dutIdx, regAddr, byteOffset, readCount, regValueByteArray, readCount)
+        if ret==1 :
+            for idx, regValueByte in enumerate(regValueByteArray) :
+                regValueInt = regValueByte
+                regValueInt &= 0xFF
+                regValueList[idx] = regValueInt
+        
+        return regValueList
+
+    #TDEVICE_API unsigned char TDeviceDD_DSIM_MipiReadReg1Byte(void* hdl, int dutIdx, int addr, int byteOffset);
     def DD_DSIM_MipiReadReg1Byte(this, dutIdx, regAddr, byteOffset) : 
-        return 1 #ted.DD_DSIM_MipiReadReg1Byte(dutIdx, regAddr, byteOffset)
+        return this.__Api.TDeviceDD_DSIM_MipiReadReg1Byte(this.__DeviceHandle,  dutIdx, regAddr, byteOffset)
 
-    #COMM_API Bool TedDD_DSIM_MipiReadReg(int dutIdx, int addr, int byteOffset, int readCount, unsigned char* buf, int bufMaxByteSize);
+    #TDEVICE_API TED_BOOL TDeviceDD_DSIM_MipiWriteReg(void* hdl, int dutIdx, int addr, int byteOffset, int writeCount, const unsigned char* buf);
     def DD_DSIM_MipiWriteReg(this, dutIdx, regAddr, byteOffset, regValueList) : 
-        return 1 #ted.DD_DSIM_MipiWriteReg(dutIdx, regAddr, byteOffset, regValueList)
+        writeCount = len(regValueList)
+        regValueByteArray=bytes(regValueList)
+        ret = this.__Api.TDeviceDD_DSIM_MipiWriteReg(this.__DeviceHandle, dutIdx, regAddr, byteOffset, writeCount, regValueByteArray)
+        if ret==1 :
+            bflag = True
+        else :
+            bflag = False;
+        return bflag
+
 
     #COMM_API Bool TedDD_DSIM_MipiWriteReg1Byte(int dutIdx, int addr, int byteOffset, unsigned char data);
     def DD_DSIM_MipiWriteReg1Byte(this, dutIdx, regAddr, byteOffset, regValue) : 
-        return 1 #ted.DD_DSIM_MipiWriteReg1Byte(dutIdx, regAddr, byteOffset, regValue)
+        return this.__Api.TDeviceDD_DSIM_MipiReadReg1Byte(this.__DeviceHandle,  dutIdx, regAddr, byteOffset, regValue)
 
     #WREG0=0x39, [Addr], [regVal0], [regVal1].....
     def DD_DSIM_MipiWriteReg39(this, dutIdx, regAddr, regValueList) :
         #print("[this.__Api.TDevice.DD_DSIM_MipiWriteReg39] 0x%02x" % (regAddr))
         regValueByteArray=bytes(regValueList)
-        ret = this.__Api.TDeviceDD_DSIM_MipiWriteReg39(this.__DeviceHandle, regAddr, len(regValueList), regValueByteArray)
+        ret = this.__Api.TDeviceDD_DSIM_MipiWriteReg39(this.__DeviceHandle, dutIdx, regAddr, len(regValueList), regValueByteArray)
         if ret==1 :
             bflag = True
         else :
@@ -667,10 +678,7 @@ class TDeviceWin(TDeviceBase) :
 
     #WREG0=0x15, [Addr], [regVal]
     def DD_DSIM_MipiWriteReg15(this, dutIdx, regAddr, regValue) :
-
-        #print("[this.__Api.TDevice.DD_DSIM_MipiWriteReg15] 0x%02x" % (regAddr))
-
-        ret = this.__Api.TDeviceDD_DSIM_MipiWriteReg15(this.__DeviceHandle, regAddr, regValue)
+        ret = this.__Api.TDeviceDD_DSIM_MipiWriteReg15(this.__DeviceHandle, dutIdx, regAddr, regValue)
         if ret==1 :
             bflag = True
         else :
@@ -679,10 +687,7 @@ class TDeviceWin(TDeviceBase) :
 
     #WREG0=0x05, [Addr]
     def DD_DSIM_MipiWriteReg05(this, dutIdx, regAddr) :
-
-        #print("[this.__Api.TDevice.DD_DSIM_MipiWriteReg05] 0x%02x" % (regAddr))
-
-        ret = this.__Api.TDeviceDD_DSIM_MipiWriteReg05(this.__DeviceHandle, regAddr)
+        ret = this.__Api.TDeviceDD_DSIM_MipiWriteReg05(this.__DeviceHandle, dutidx, regAddr)
         if ret==1 :
             bflag = True
         else :
@@ -691,7 +696,7 @@ class TDeviceWin(TDeviceBase) :
 
     #WREG0=0x07, [value]   :   Compressd Mode Command
     def DD_DSIM_MipiWriteReg07(this, dutIdx, value) :
-        ret = this.__Api.TDeviceDD_DSIM_MipiWriteReg07(this.__DeviceHandle, value)
+        ret = this.__Api.TDeviceDD_DSIM_MipiWriteReg07(this.__DeviceHandle, dutIdx, value)
         if ret==1 :
             bflag = True
         else :
@@ -987,7 +992,17 @@ class TDeviceWin(TDeviceBase) :
 
     #COMM_API Bool TedAdcGetAllCurrent(int dutIdx, int groupIdx, int* valueArray);
     def AdcGetAllCurrent(this, dutIdx, grpIdx) :
-        return this.__Api.TDeviceAdcGetAllCurrent(dutIdx, grpIdx)
+        
+        chCnt = this.AdcGetChannelCount(dutIdx, grpIdx)
+        adcValueArrayBytes = bytes(chCnt*4)
+        this.__Api.TDeviceAdcGetAllCurrent(this.__DeviceHandle, dutIdx, grpIdx, adcValueArrayBytes)
+        
+        unPackResult= struct.unpack(("%di" % chCnt), adcValueArrayBytes)
+        adcList = [0 for _ in range(chCnt)]
+        for i in range(chCnt) :
+            adcList[i] = unPackResult[i]
+
+        return adcList
 
     #COMM_API Bool TedAdcSoutSetRBSel(int dutIdx, int value);
     def AdcSoutSetRBSel(this, dutIdx, value) :
